//ChatServer.cc
#include "ChatServer.h"
#include <iostream>
using namespace std;

ChatServer * ClientHandler::ch_server = 0;
ChatServer::ChatServer( )
{
  cs_server = new QTcpServer();
  if (! cs_server->listen()) 
    {
      qWarning( "Failed to register the server port" );
      exit( 1 );
    }
  cout << "Server port " << cs_server->serverPort() << endl;
  connect(cs_server, SIGNAL(newConnection()), 
	  this, SLOT(connectNewClient()));
  ClientHandler::ch_server = this;
}

void ChatServer::connectNewClient() 
{
  QTcpSocket* socket = cs_server->nextPendingConnection();
  ClientHandler* clh = new ClientHandler( socket );
  cs_clientList.push_back(clh);
  cout << "A new client connected " << endl;
}

void ChatServer::broadcastClient(QString message)
{
  ClientHandler * handler;
  for (int index = 0; index < cs_clientList.size(); index ++)
    {
      handler = cs_clientList[index];
      *(handler-> ch_os) << message;
      (handler-> ch_os) -> flush();
    }

  // A chatter's terminal always shows name at beginning
  // of a new line.  
  for (int index = 0; index < cs_clientList.size(); index ++)
    {
      ClientHandler * handler = cs_clientList[index];
      *(handler-> ch_os) << (handler->ch_name + ": ");
      (handler-> ch_os) -> flush();
    }
}

ChatServer::~ChatServer()
{
  if (cs_server) 
    { delete cs_server; }
  ClientHandler* clh = cs_clientList.takeFirst();
  while (clh != 0)
    {
      delete clh;
      clh = cs_clientList.takeFirst();
    }
}

ClientHandler::ClientHandler( QTcpSocket* socket)
  : ch_name(""), 
    ch_socket( socket )
{
  ch_os = new QTextStream( ch_socket );

  (*ch_os) << "Welcome to a chat room powered by C++\n";      
  (*ch_os) << ">>>>    Enter 'bye' to exit   <<<\n";      
  (*ch_os) << "Enter chat name: ";      
  ch_os -> flush();
  connect( ch_socket, SIGNAL( readyRead() ),
	   this, SLOT( readFromClient() ) );
}

ClientHandler::~ClientHandler()
{
  if (ch_os) 
    { delete ch_os; }
}

void ClientHandler::readFromClient() {
  QTcpSocket* sock = (QTcpSocket*) sender();
  while ( sock->canReadLine() ) {
    QString qstr = sock->readLine();
    qstr = qstr.trimmed(); // remove white space
    if ( ch_name == "" ) 
      {
	ch_name = qstr;
	QString outgoing = "\nMessage from chat server: " + 
	  ch_name + " signed in\n";
	ch_server -> broadcastClient(outgoing);
      }
    else if ( qstr == "bye" ) 
      {
	QString outgoing = "\nMessage from the chat server: " +
	  ch_name + " signed off\n";
	ch_server -> broadcastClient(outgoing);
	ch_socket->close();
	ch_socket = 0;
      }
    else 
      {
	QString outgoing = "\n" + ch_name + ": " + qstr + "\n";
	ch_server -> broadcastClient(outgoing);
      }
  }
}


int main( int argc, char* argv[] )
{
  QApplication app( argc, argv );
  ChatServer server;
  return app.exec();
}
