//PrimeServer.cc
#include "PrimeServer.h"
#include "PrimeNumber.h"
#include <iostream>
using namespace std;

PrimeServer::PrimeServer( )
{
  cs_server = new QTcpServer();
  if (! cs_server->listen()) 
    {
      qWarning( "Failed to register the server port" );
      exit( 1 );
    }
  cout << "Server port " << cs_server->serverPort() << endl;
  connect(cs_server, SIGNAL(newConnection()), 
	  this, SLOT(connectNewClient()));
}

void PrimeServer::connectNewClient() 
{
  QTcpSocket* socket = cs_server->nextPendingConnection();
  ClientHandler* clh = new ClientHandler( socket );
  cs_clientList.push_back(clh);
  cout << "A new client connected " << endl;
}

PrimeServer::~PrimeServer()
{
  if (cs_server) 
    { delete cs_server; }
  ClientHandler* clh = cs_clientList.takeFirst();
  while (clh != 0)
    {
      delete clh;
      clh = cs_clientList.takeFirst();
    }
}

ClientHandler::ClientHandler( QTcpSocket* socket)
  : ch_socket( socket )
{
  ch_os = new QTextStream( ch_socket );

  (*ch_os) << "Welcome to a prime number counter\n";      
  (*ch_os) << ">>>>    Enter 'bye' to exit   <<<\n";      
  (*ch_os) << "Give a number: ";      
  ch_os -> flush();
  connect( ch_socket, SIGNAL( readyRead() ),
	   this, SLOT( readFromClient() ) );
}

ClientHandler::~ClientHandler()
{
  if (ch_os) 
    { delete ch_os; }
}

void ClientHandler::readFromClient() {
  QTcpSocket* sock = (QTcpSocket*) sender();
  while ( sock->canReadLine() ) {
    QString qstr = sock->readLine();
    qstr = qstr.trimmed(); 
    if ( qstr == "bye" ) 
      {
	(*ch_os) << "Bye\n\n";
	ch_os -> flush();
	ch_socket->close();
	ch_socket = 0;
      }
    else 
      {
	long maxValue = qstr.toLong();
	long total = TotalPrime(maxValue);
	QString outgoing = "\n There are " + QString::number(total) + 
	  " prime numbers between 0 and " + QString::number(maxValue)
	  + "\n";
	(*ch_os) << outgoing;
	(*ch_os) << "Give a number or say \"bye\": ";      
	ch_os -> flush();
      }
  }
}


int main( int argc, char* argv[] )
{
  QApplication app( argc, argv );
  PrimeServer server;
  return app.exec();
}
